import json, requests
from urllib import splitquery
from urlparse import urlparse as parse


class Collection(object):
    model = None

    def __init__(self, client):
        self.client = client

    def handle_error(self, response):
        import logging
        logger = logging.getLogger('dj_capysule')
        logger.error(response.text)
        response.raise_for_status()

    def decode(self, data, many=False):
        if many:
            result = []
            for d in data:
                obj = self.model(**self.model.decode(d))
                obj._persisted = True
                result.append(obj)
            return result
        else:
            return self.model(**self.model.decode(data))

    def encode(self, obj):
        return obj.encode()

    def all(self):
        response = self.client.fetch(self.url)

        if response.status_code >= 400:
            self.handle_error(response)

        data = response.json()

        return self.decode(data, many=True)

    def query(self, **kwargs):
        request = requests.Request('GET', self.url,
            params=kwargs,
            headers={'Content-Type': 'application/json'}
        )
        response = self.client.fetch(request)

        if response.status_code >= 400:
            self.handle_error(response)

        data = response.json()

        return self.decode(data, many=True)

    def get(self, id_):
        response = self.client.fetch(self._url(id_))

        if response.status_code >= 400:
            self.handle_error(response)

        data = response.json()

        return self.decode(data)

    def _url(self, id_):
        url = self.url

        if callable(url):
           return url(id_)

        url, query = splitquery(url)

        url = '{0}/{1}'.format(url, id_)

        if query is not None:
            url = '{0}?{1}'.format(url, query)

        return url

    def _parse_url(self, url):
        parse_url = getattr(self.model, '_parse_url', None)

        if callable(parse_url):
            return parse_url(url)

        parts = url.split('/')
        if len(parts):
            return parts[-1]
        else:
            return None

    def add(self, obj):
        if getattr(obj, '_persisted', False) is True:
            url = self._url(self._id(obj))
            method = 'PUT'
        else:
            url = getattr(obj, '_url', self.url)
            if callable(url):
                url = url()
            method = 'POST'

        data = self.encode(obj)

        request = requests.Request(method, url,
            data=json.dumps(data),
            headers={'Content-Type': 'application/json'}
        )

        response = self.client.fetch(request)

        if response.status_code >= 400:
            self.handle_error(response)

        if len(response.content) > 0:
            data = response.json()

            try:
                obj.decode(data)
                obj._persisted = True
            except Exception as error:
                raise

            return obj
        else:
            try:
                url = response.headers.get('Location')
            except KeyError:
                return None
            id_ = self._parse_url(url)
            if id_ is not None:
                for name, field in obj._fields.items():
                    if field.options.get('primary', False):
                        setattr(obj, name, id_)
                        return obj
            return None


    def _id(self, obj):
        for name, field in obj._fields.items():
            if field.options.get('primary', False):
                return getattr(obj, name)



import requests
import urlparse

class Client(object):

    def __init__(self, base_uri=None):
        self.base_uri = base_uri
        self._session = None
        self._auth = None
        self._headers = {}

    def set_basic_auth(self, user, password):
        self.session.auth = (user, password)
        self._auth = (user, password)

    def set_headers(self, headers):
        self.session.headers.update(headers)
        self._headers.update(headers)

    @property
    def session(self):
        if self._session is None:
            self._session = requests.Session()
        return self._session

    def log_request(self, request):
        import logging
        logger = logging.getLogger('dj_capysule')
        logger.debug('-------------------')
        if isinstance(request, basestring):
            logger.debug('GET %s' % request)
        else:
            logger.debug('%s %s' % (request.method, request.url))
            logger.debug('Headers')
            logger.debug(request.headers)
            logger.debug('Params')
            logger.debug(request.params)
            logger.debug('Data')
            logger.debug(request.data)
            logger.debug('Auth')
            logger.debug(request.auth)
        logger.debug('-------------------')

    def log_response(self, response):
        import logging
        logger = logging.getLogger('dj_capysule')
        logger.debug('===================')
        logger.debug(response.text)
        logger.debug('===================')

    def fetch(self, request):
        if isinstance(request, basestring):
            joined = urlparse.urljoin(self.base_uri, request)
            self.log_request(joined)
            response = self.session.get(joined)
        elif isinstance(request, requests.Request):
            request.url = urlparse.urljoin(self.base_uri, request.url)
            request.auth = self.session.auth
            headers = self._headers
            headers.update(request.headers)
            request.headers = headers
            self.log_request(request)
            prepared = request.prepare()
            response = self.session.send(prepared)
        else:
            raise TypeError('Request should be an instance of request.Request or a string')
        self.log_response(response)
        return response



version = '0.3.4'

# https://coderwall.com/p/qawuyq
# Thanks James.

try:
   import pypandoc
   long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError, RuntimeError):
   long_description = ''

import os
requirements_file = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'requirements.txt')

with open(requirements_file, 'r') as f:
    install_requires = [x.strip() for x in f.readlines()]

from setuptools import setup

setup(
    name='wren',
    version=version,
    author='Body Labs',
    author_email='paul.melnikow@bodylabs.com',
    description='Synchronous RESTful API consumer based on Requests',
    long_description=long_description,
    url='https://github.com/bodylabs/wren',
    license='Apache 2',
    packages=[
        'wren',
    ],
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
